"""Defines outputs supported by STACS.

SPDX-License-Identifier: BSD-3-Clause
"""

from stacs.scan.output import markdown  # noqa: F401
from stacs.scan.output import pretty  # noqa: F401
from stacs.scan.output import sarif  # noqa: F401
